@@ -1,7 +1,7 @@
-/* $Id: cidr.c 2284 2009-05-03 07:24:10Z aturner $ */
+/* $Id: cidr.c 2423 2010-03-13 07:09:49Z aturner $ */
 
 /*
- * Copyright (c) 2001-2004 Aaron Turner.
+ * Copyright (c) 2001-2010 Aaron Turner.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -230,6 +230,10 @@ cidr2cidr(char *cidr)
         if (newcidr->masklen < 0 || newcidr->masklen > 128)
             goto error;
 
+        /* skip past the opening [ */
+        if (*cidr == '[')
+            cidr ++;
+
         if (get_name2addr6(cidr, RESOLVE, &newcidr->u.network6) > 0) {
             family = AF_INET6;
         } else {
