@@ -20,11 +20,11 @@
 #endif
 
 #if defined INCLUDE_PCAP_BPF_H_FILE && !defined PCAP_DONT_INCLUDE_PCAP_BPF_H
-#include "/opt/local/include/pcap/bpf.h"
+#include "/usr/local/include/pcap/bpf.h"
 #define PCAP_DONT_INCLUDE_PCAP_BPF_H 1 /* don't re-include it in pcap.h */
 #endif
 
-#include "/opt/local/include/pcap.h"
+#include "/usr/local/include/pcap.h"
 
 #include "lib/strlcpy.h"
 #include "common/list.h"
