@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: mod_ip_tos.c 2000 2008-04-27 06:17:35Z aturner $
+ * $Id: mod_ip_tos.c 2303 2009-05-06 18:48:20Z aturner $
  */
 
 #include "config.h"
@@ -53,10 +53,14 @@ ip_tos_apply(void *d, struct pktq *pktq)
 	struct pkt *pkt;
 
 	TAILQ_FOREACH(pkt, pktq, pkt_next) {
+		uint16_t eth_type = htons(pkt->pkt_eth->eth_type);
+
+		if (eth_type == ETH_TYPE_IP) {
 		pkt->pkt_ip->ip_tos = data->tos;
 		/* XXX - do incremental checksum */
 		ip_checksum(pkt->pkt_ip, pkt->pkt_ip_data - pkt->pkt_eth_data);
 	}
+	}
 	return (0);
 }
 
