@@ -1,7 +1,7 @@
-/* $Id: send_packets.c 2346 2009-05-19 00:14:52Z aturner $ */
+/* $Id: send_packets.c 2444 2010-03-30 04:47:09Z aturner $ */
 
 /*
- * Copyright (c) 2001-2008 Aaron Turner.
+ * Copyright (c) 2001-2010 Aaron Turner.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -72,8 +72,6 @@ extern int debug;
 
 static void do_sleep(struct timeval *time, struct timeval *last, int len, 
     int accurate, sendpacket_t *sp, COUNTER counter, delta_t *ctx);
-static const u_char *get_next_packet(pcap_t *pcap, struct pcap_pkthdr *pkthdr, 
-    int file_idx, packet_cache_t **prev_packet);
 static u_int32_t get_user_count(sendpacket_t *sp, COUNTER counter);
 
 /**
@@ -83,21 +81,21 @@ static u_int32_t get_user_count(sendpacket_t *sp, COUNTER counter);
 void
 send_packets(pcap_t *pcap, int cache_file_idx)
 {
-    struct timeval last = { 0, 0 };
+    struct timeval last = { 0, 0 }, last_print_time = { 0, 0 }, print_delta, now;
     COUNTER packetnum = 0;
     struct pcap_pkthdr pkthdr;
     const u_char *pktdata = NULL;
     sendpacket_t *sp = options.intf1;
     u_int32_t pktlen;
-	packet_cache_t *cached_packet = NULL;
-	packet_cache_t **prev_packet = NULL;
+    packet_cache_t *cached_packet = NULL;
+    packet_cache_t **prev_packet = NULL;
 #if defined TCPREPLAY && defined TCPREPLAY_EDIT
     struct pcap_pkthdr *pkthdr_ptr;
 #endif
     delta_t delta_ctx;
 
     init_delta_time(&delta_ctx);
-    
+
     /* register signals */
     didsig = 0;
     if (options.speed.mode != SPEED_ONEATATIME) {
@@ -112,7 +110,7 @@ send_packets(pcap_t *pcap, int cache_file_idx)
         prev_packet = NULL;
     }
 
-	
+
     /* MAIN LOOP 
      * Keep sending while we have packets or until
      * we've sent enough packets
@@ -138,24 +136,24 @@ send_packets(pcap_t *pcap, int cache_file_idx)
 #endif
 
         dbgx(2, "packet " COUNTER_SPEC " caplen %d", packetnum, pktlen);
-        
+
         /* Dual nic processing */
         if (options.intf2 != NULL) {
 
             sp = (sendpacket_t *) cache_mode(options.cachedata, packetnum);
-        
+
             /* sometimes we should not send the packet */
             if (sp == TCPR_DIR_NOSEND)
                 continue;
         }
-    
+
         /* do we need to print the packet via tcpdump? */
 #ifdef ENABLE_VERBOSE
         if (options.verbose)
             tcpdump_print(options.tcpdump, &pkthdr, pktdata);
 #endif
 
-#if defined TCPREPLAY && defined TCPREPLAY_EDIT        
+#if defined TCPREPLAY && defined TCPREPLAY_EDIT
         pkthdr_ptr = &pkthdr;
         if (tcpedit_packet(tcpedit, &pkthdr_ptr, &pktdata, sp->cache_dir) == -1) {
             errx(-1, "Error editing packet #" COUNTER_SPEC ": %s", packetnum, tcpedit_geterr(tcpedit));
@@ -174,15 +172,15 @@ send_packets(pcap_t *pcap, int cache_file_idx)
         /* mark the time when we send the last packet */
         start_delta_time(&delta_ctx);
         dbgx(2, "Sending packet #" COUNTER_SPEC, packetnum);
-        
+
         /* write packet out on network */
         if (sendpacket(sp, pktdata, pktlen) < (int)pktlen)
             warnx("Unable to send packet: %s", sendpacket_geterr(sp));
 
-        /* 
+        /*
          * track the time of the "last packet sent".  Again, because of OpenBSD
-         * we have to do a mempcy rather then assignment. 
-         * 
+         * we have to do a mempcy rather then assignment.
+         *
          * A number of 3rd party tools generate bad timestamps which go backwards
          * in time.  Hence, don't update the "last" unless pkthdr.ts > last
          */
@@ -190,95 +188,111 @@ send_packets(pcap_t *pcap, int cache_file_idx)
             memcpy(&last, &pkthdr.ts, sizeof(struct timeval));
         pkts_sent ++;
         bytes_sent += pktlen;
+
+        /* print stats during the run? */
+        if (options.stats > 0) {
+            if (gettimeofday(&now, NULL) < 0)
+                errx(-1, "gettimeofday() failed: %s",  strerror(errno));
+
+            if (! timerisset(&last_print_time)) {
+                memcpy(&last_print_time, &now, sizeof(struct timeval));
+            } else {
+                timersub(&now, &last_print_time, &print_delta);
+                if (print_delta.tv_sec >= options.stats) {
+                    packet_stats(&begin, &now, bytes_sent, pkts_sent, failed);
+                    memcpy(&last_print_time, &now, sizeof(struct timeval));
+                }
+            }
+        }
     } /* while */
 
-	if (options.enable_file_cache) {
-		options.file_cache[cache_file_idx].cached = TRUE;
-	}
+    if (options.enable_file_cache) {
+        options.file_cache[cache_file_idx].cached = TRUE;
+    }
 }
 
 /**
  * Gets the next packet to be sent out. This will either read from the pcap file
  * or will retrieve the packet from the internal cache.
- *	
+ *
  * The parameter prev_packet is used as the parent of the new entry in the cache list.
  * This should be NULL on the first call to this function for each file and
  * will be updated as new entries are added (or retrieved) from the cache list.
  */
-static const u_char *
+const u_char *
 get_next_packet(pcap_t *pcap, struct pcap_pkthdr *pkthdr, int file_idx, 
     packet_cache_t **prev_packet)
 {
-	u_char *pktdata = NULL;
+    u_char *pktdata = NULL;
     u_int32_t pktlen;
 
     /* pcap may be null in cache mode! */
     /* packet_cache_t may be null in file read mode! */
     assert(pkthdr);
 
-	/*
-	 * Check if we're caching files
-	 */
-	if (options.enable_file_cache && (prev_packet != NULL)) {
-		/*
-		 * Yes we are caching files - has this one been cached?
-		 */
-		if (options.file_cache[file_idx].cached) {
-			if (*prev_packet == NULL) {
-				/*
-				 * Get the first packet in the cache list directly from the file
-				 */
-				*prev_packet = options.file_cache[file_idx].packet_cache;
-			} else {
-				/*
-				 * Get the next packet in the cache list
-				 */
-				*prev_packet = (*prev_packet)->next;
-			}
-			
-			if (*prev_packet != NULL) {
-				pktdata = (*prev_packet)->pktdata;
-				memcpy(pkthdr, &((*prev_packet)->pkthdr), sizeof(struct pcap_pkthdr));
-			}
-		} else {
-			/*
-			 * We should read the pcap file, and cache the results
-			 */
-			pktdata = (u_char *)pcap_next(pcap, pkthdr);
-			if (pktdata != NULL) {
-				if (*prev_packet == NULL) {
-					/*
-					 * Create the first packet in the list
-					 */
-					*prev_packet = safe_malloc(sizeof(packet_cache_t));
-					options.file_cache[file_idx].packet_cache = *prev_packet;
-				} else {
-					/*
-					 * Add a packet to the end of the list
-					 */
-					(*prev_packet)->next = safe_malloc(sizeof(packet_cache_t));
-					*prev_packet = (*prev_packet)->next;
-				}
-				
-				if (*prev_packet != NULL) {
-					(*prev_packet)->next = NULL;
-					pktlen = pkthdr->len;
-					
-					(*prev_packet)->pktdata = safe_malloc(pktlen);
-					memcpy((*prev_packet)->pktdata, pktdata, pktlen);
-					memcpy(&((*prev_packet)->pkthdr), pkthdr, sizeof(struct pcap_pkthdr));
-				}
-			}
-		}
-	} else {
-		/*
-		 * Read pcap file as normal
-		 */
-		pktdata = (u_char *)pcap_next(pcap, pkthdr);
-	}
+    /*
+     * Check if we're caching files
+     */
+    if ((options.enable_file_cache || options.preload_pcap) && (prev_packet != NULL)) {
+        /*
+         * Yes we are caching files - has this one been cached?
+         */
+        if (options.file_cache[file_idx].cached) {
+            if (*prev_packet == NULL) {
+                /*
+                 * Get the first packet in the cache list directly from the file
+                 */
+                *prev_packet = options.file_cache[file_idx].packet_cache;
+            } else {
+                /*
+                 * Get the next packet in the cache list
+                 */
+                *prev_packet = (*prev_packet)->next;
+            }
+
+            if (*prev_packet != NULL) {
+                pktdata = (*prev_packet)->pktdata;
+                memcpy(pkthdr, &((*prev_packet)->pkthdr), sizeof(struct pcap_pkthdr));
+            }
+        } else {
+            /*
+             * We should read the pcap file, and cache the results
+             */
+            pktdata = (u_char *)pcap_next(pcap, pkthdr);
+            if (pktdata != NULL) {
+                if (*prev_packet == NULL) {
+                    /*
+                     * Create the first packet in the list
+                     */
+                    *prev_packet = safe_malloc(sizeof(packet_cache_t));
+                    options.file_cache[file_idx].packet_cache = *prev_packet;
+                } else {
+                    /*
+                     * Add a packet to the end of the list
+                     */
+                    (*prev_packet)->next = safe_malloc(sizeof(packet_cache_t));
+                    *prev_packet = (*prev_packet)->next;
+                }
+
+                if (*prev_packet != NULL) {
+                    (*prev_packet)->next = NULL;
+                    pktlen = pkthdr->len;
+
+                    (*prev_packet)->pktdata = safe_malloc(pktlen);
+                    memcpy((*prev_packet)->pktdata, pktdata, pktlen);
+                    memcpy(&((*prev_packet)->pkthdr), pkthdr, sizeof(struct pcap_pkthdr));
+                }
+            }
+        }
+    } else {
+        /*
+         * Read pcap file as normal
+         */
+        pktdata = (u_char *)pcap_next(pcap, pkthdr);
+    }
 
     /* this get's casted to a const on the way out */
-	return pktdata;
+    return pktdata;
 }
 
 /**
@@ -337,7 +351,7 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
     static int32_t nsec_adjuster = -1, nsec_times = -1;
     float n;
     static u_int32_t send = 0;      /* accellerator.   # of packets to send w/o sleeping */
-    u_int64_t ppnsec; /* packets per usec */
+    u_int32_t ppnsec;               /* packets per usec */
     static int first_time = 1;      /* need to track the first time through for the pps accelerator */
 
 
@@ -347,14 +361,14 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
 #else
     adjuster.tv_nsec = 0;
 #endif
-    
+
     /* acclerator time? */
     if (send > 0) {
         send --;
         return;
     }
-    
-    /* 
+
+    /*
      * pps_multi accelerator.    This uses the existing send accelerator above
      * and hence requires the funky math to get the expected timings.
      */
@@ -365,7 +379,7 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
             return;
         }
     }
-        
+
     dbgx(4, "This packet time: " TIMEVAL_FORMAT, time->tv_sec, time->tv_usec);
     dbgx(4, "Last packet time: " TIMEVAL_FORMAT, last->tv_sec, last->tv_usec);
 
@@ -387,9 +401,12 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
     /* If top speed, you shouldn't even be here */
     assert(options.speed.mode != SPEED_TOPSPEED);
 
+    /*
+     * 1. First, figure out how long we should sleep for...
+     */
     switch(options.speed.mode) {
     case SPEED_MULTIPLIER:
-        /* 
+        /*
          * Replay packets a factor of the time they were originally sent.
          */
         if (timerisset(last)) {
@@ -401,7 +418,7 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
                 /* time has increased or is the same, so handle normally */
                 timersub(time, last, &nap_for);
                 dbgx(3, "original packet delta time: " TIMEVAL_FORMAT, nap_for.tv_sec, nap_for.tv_usec);
-                
+
                 TIMEVAL_TO_TIMESPEC(&nap_for, &nap);
                 dbgx(3, "original packet delta timv: " TIMESPEC_FORMAT, nap.tv_sec, nap.tv_nsec);
                 timesdiv(&nap, options.speed.speed);
@@ -410,11 +427,11 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
         } else {
             /* Don't sleep if this is our first packet */
             timesclear(&nap);
-        }        
+        }
         break;
 
     case SPEED_MBPSRATE:
-        /* 
+        /*
          * Ignore the time supplied by the capture file and send data at
          * a constant 'rate' (bytes per second).
          */
@@ -422,7 +439,7 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
             n = (float)len / (options.speed.speed * 1024 * 1024 / 8); /* convert Mbps to bps */
             nap.tv_sec = n;
             nap.tv_nsec = (n - nap.tv_sec)  * 1000000000;
-            
+
             dbgx(3, "packet size %d\t\tequals %f bps\t\tnap " TIMESPEC_FORMAT, len, n, 
                 nap.tv_sec, nap.tv_nsec);
         }
@@ -433,16 +450,23 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
         break;
 
     case SPEED_PACKETRATE:
-        /* only need to calculate this the first time */
+        /*
+         * Only need to calculate this the first time since this is a
+         * constant time function
+         */
         if (! timesisset(&nap)) {
             /* run in packets/sec */
             ppnsec = 1000000000 / options.speed.speed * (options.speed.pps_multi > 0 ? options.speed.pps_multi : 1);
             NANOSEC_TO_TIMESPEC(ppnsec, &nap);
             dbgx(1, "sending %d packet(s) per %lu nsec", (options.speed.pps_multi > 0 ? options.speed.pps_multi : 1), nap.tv_nsec);
-        }        
+        }
         break;
-        
+
     case SPEED_ONEATATIME:
+        /*
+         * Prompt the user for sending each packet(s)
+         */
+
         /* do we skip prompting for a key press? */
         if (send == 0) {
             send = get_user_count(sp, counter);
@@ -453,8 +477,7 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
                sp == options.intf1 ? options.intf1_name : options.intf2_name);
         send --;
 
-        /* leave do_sleep() */
-        return;
+        return; /* leave do_sleep() */
 
         break;
 
@@ -463,11 +486,10 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
         break;
     }
 
-    /* 
+    /*
      * since we apply the adjuster to the sleep time, we can't modify nap
      */
-    nap_this_time.tv_sec = nap.tv_sec;
-    nap_this_time.tv_nsec = nap.tv_nsec;
+    memcpy(&nap_this_time, &nap, sizeof(nap_this_time));
 
     dbgx(2, "nap_time before rounding:   " TIMESPEC_FORMAT, nap_this_time.tv_sec, nap_this_time.tv_nsec);
 
@@ -475,30 +497,33 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
     if (accurate != ACCURATE_ABS_TIME) {
 
         switch (options.speed.mode) {
-            /* Mbps & Multipler are dynamic timings, so we round to the nearest usec */
+            /*
+             * We used to round to the nearest uset for Mbps & Multipler 
+             * because they are "dynamic timings", but that seems stupid
+             * so I'm turning that off and we do nothing now
+             */
             case SPEED_MBPSRATE:
             case SPEED_MULTIPLIER:
-                ROUND_TIMESPEC_TO_MICROSEC(&nap_this_time);
                 break;
 
             /* Packets/sec is static, so we weight packets for .1usec accuracy */
             case SPEED_PACKETRATE:
                 if (nsec_adjuster < 0)
                     nsec_adjuster = (nap_this_time.tv_nsec % 10000) / 1000;
-        
+
                 /* update in the range of 0-9 */
                 nsec_times = (nsec_times + 1) % 10;
-        
+
                 if (nsec_times < nsec_adjuster) {
                     /* sorta looks like a no-op, but gives us a nice round usec number */
                     nap_this_time.tv_nsec = (nap_this_time.tv_nsec / 1000 * 1000) + 1000;
                 } else {
                     nap_this_time.tv_nsec -= (nap_this_time.tv_nsec % 1000);
                 }
-    
+
                 dbgx(3, "(%d)\tnsec_times = %d\tnap adjust: %lu -> %lu", nsec_adjuster, nsec_times, nap.tv_nsec, nap_this_time.tv_nsec);            
                 break;
-                
+
             default:
                 errx(-1, "Unknown/supported speed mode: %d", options.speed.mode);
         }
@@ -526,13 +551,13 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
             timesclear(&nap_this_time);
         }
     }
-    
+
     dbgx(2, "Sleeping:                   " TIMESPEC_FORMAT, nap_this_time.tv_sec, nap_this_time.tv_nsec);
 
     /* don't sleep if nap = {0, 0} */
     if (!timesisset(&nap_this_time))
         return;
-        
+
     /*
      * Depending on the accurate method & packet rate computation method
      * We have multiple methods of sleeping, pick the right one...
@@ -550,7 +575,7 @@ do_sleep(struct timeval *time, struct timeval *last, int len, int accurate,
         break;
 #endif
 
-#ifdef HAVE_RDTSC        
+#ifdef HAVE_RDTSC
     case ACCURATE_RDTSC:
         rdtsc_sleep(nap_this_time);
         break;
