@@ -1,7 +1,7 @@
-/* $Id: en10mb.c 2105 2009-01-07 05:48:32Z aturner $ */
+/* $Id: en10mb.c 2423 2010-03-13 07:09:49Z aturner $ */
 
 /*
- * Copyright (c) 2006-2007 Aaron Turner.
+ * Copyright (c) 2006-2010 Aaron Turner.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -343,9 +343,15 @@ dlt_en10mb_encode(tcpeditdlt_t *ctx, u_char *packet, int pktlen, tcpr_dir_t dir)
     int newl2len = 0;
 
     assert(ctx);
-    assert(pktlen >= 14);
     assert(packet);
 
+    if (pktlen < 14) {
+        tcpedit_seterr(ctx->tcpedit, 
+                "Unable to process packet #" COUNTER_SPEC " since it is less then 14 bytes.", 
+                ctx->tcpedit->runtime.packetnum);
+        return TCPEDIT_ERROR;
+    }
+
     plugin = tcpedit_dlt_getplugin(ctx, dlt_value);
     config = plugin->config;
     extra = (en10mb_extra_t *)ctx->decoded_extra;
