@@ -1,7 +1,7 @@
-/* $Id: tcprewrite.c 2105 2009-01-07 05:48:32Z aturner $ */
+/* $Id: tcprewrite.c 2427 2010-03-25 00:38:13Z aturner $ */
 
 /*
- * Copyright (c) 2004-2007 Aaron Turner.
+ * Copyright (c) 2004-2010 Aaron Turner.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -89,12 +89,12 @@ main(int argc, char *argv[])
 
     /* parse the tcprewrite args */
     post_args(argc, argv);
-    
+
     /* init tcpedit context */
     if (tcpedit_init(&tcpedit, pcap_datalink(options.pin)) < 0) {
         errx(-1, "Error initializing tcpedit: %s", tcpedit_geterr(tcpedit));
     }
-    
+
     /* parse the tcpedit args */
     rcode = tcpedit_post_args(&tcpedit);
     if (rcode < 0) {
@@ -108,7 +108,7 @@ main(int argc, char *argv[])
         errx(-1, "Unable to edit packets given options:\n%s",
                 tcpedit_geterr(tcpedit));
     }
-    
+
    /* open up the output file */
     options.outfile = safe_strdup(OPT_ARG(OUTFILE));
     dbgx(1, "Rewriting DLT to %s",
@@ -165,7 +165,7 @@ tcprewrite_init(void)
     /* clear out tcpdump struct */
     memset(&tcpdump, '\0', sizeof(tcpdump_t));
 #endif
-    
+
     if (fcntl(STDERR_FILENO, F_SETFL, O_NONBLOCK) < 0)
         warnx("Unable to set STDERR to non-blocking: %s", strerror(errno));
 }
@@ -177,7 +177,7 @@ void
 post_args(_U_ int argc, _U_ char *argv[])
 {
     char ebuf[PCAP_ERRBUF_SIZE];
-     
+
 #ifdef DEBUG
     if (HAVE_OPT(DBUG))
         debug = OPT_VALUE_DBUG;
@@ -185,21 +185,21 @@ post_args(_U_ int argc, _U_ char *argv[])
     if (HAVE_OPT(DBUG))
         warn("not configured with --enable-debug.  Debugging disabled.");
 #endif
-    
+
 
 #ifdef ENABLE_VERBOSE
     if (HAVE_OPT(VERBOSE))
         options.verbose = 1;
-    
+
     if (HAVE_OPT(DECODE))
-        tcpdump.args = safe_strdup(OPT_ARG(DECODE));    
+        tcpdump.args = safe_strdup(OPT_ARG(DECODE));
 #endif
 
 
 #ifdef ENABLE_FRAGROUTE
     if (HAVE_OPT(FRAGROUTE))
         options.fragroute_args = safe_strdup(OPT_ARG(FRAGROUTE));
-    
+
     options.fragroute_dir = FRAGROUTE_DIR_BOTH;
     if (HAVE_OPT(FRAGDIR)) {
         if (strcmp(OPT_ARG(FRAGDIR), "c2s") == 0) {
@@ -218,6 +218,13 @@ post_args(_U_ int argc, _U_ char *argv[])
     options.infile = safe_strdup(OPT_ARG(INFILE));
     if ((options.pin = pcap_open_offline(options.infile, ebuf)) == NULL)
         errx(-1, "Unable to open input pcap file: %s", ebuf);
+
+#ifdef HAVE_PCAP_SNAPSHOT
+    if (pcap_snapshot(options.pin) < 65535)
+        warnx("%s was captured using a snaplen of %d bytes.  This may mean you have truncated packets.",
+                options.infile, pcap_snapshot(options.pin));
+#endif
+
 }
 
 /** 
@@ -234,14 +241,14 @@ rewrite_packets(tcpedit_t *tcpedit, pcap_t *pin, pcap_dumper_t *pout)
     static char *frag = NULL;
     COUNTER packetnum = 0;
     int rcode, frag_len, i;
-    
+
     pkthdr_ptr = &pkthdr;
 
     if (pktdata_buff == NULL)
         pktdata_buff = (u_char *)safe_malloc(MAXPACKET);
-        
+
     pktdata = &pktdata_buff;
-    
+
     if (frag == NULL)
         frag = (char *)safe_malloc(MAXPACKET);
 
@@ -258,7 +265,7 @@ rewrite_packets(tcpedit_t *tcpedit, pcap_t *pin, pcap_dumper_t *pout)
          * because pcap_next() returns a const ptr
          */
         memcpy(*pktdata, pktconst, pkthdr.caplen);
-        
+
 #ifdef ENABLE_VERBOSE
         if (options.verbose)
             tcpdump_print(&tcpdump, pkthdr_ptr, *pktdata);
@@ -321,7 +328,7 @@ WRITE_PACKET:
 #endif
     } /* while() */
     return 0;
-}   
+}
 
 
 /*
